#include <stdio.h>
#include <fcntl.h>
#include "hdfi.h"
#include "cris.h"
#include "FileTypes.h"
#include "crisPutData.h"
#include "crisSisPutUtilities.h"
#include "utilities.h"

/*

version 2:
	broke job up into subroutines that can be used across all of the
	putCrisData routines.  also added a new return value:

		returns 3 if data to write to file is exactly the same
			as data which is already in the file.  the file
			is not written to in this case.

version 1:

putCrisEvents is passed a pointer to a struct L1CrisEventCycle which it then
attempts to archive inside a daily file in the cris data archive directory
tree in this manner:

a base data directory is gotten from the environment variable
L1_CRIS_DATA_BASE_DIRECTORY.  if this environment variable has
not been defined, putCrisEvents exits.  for following examples,
let's say this has been defined as /ace/cris/data.  if /ace/cris/data
does not exist, putCrisEvents exits.

putCrisEvents reads the space craft seconds since 1996 and determines
the year, month, day, and seconds since midnight into the day.  for
example's sake, say year = 1999, month = 5 (1-12), day = 23 (1-31),
and seconds since midnight = 22174 (0-86399).

putCrisEvents then checks for the existence of a directory named
/ace/cris/data/C1999.05/, and if not finding it, creates it.  the
same for the next directory level, /ace/cris/data/C1999.05/C1999.05.23/

putCrisEvents then checks for the existence of the file
/ace/cris/data/C1999.05/C1999.05.23/C1999.05.23.evt
if this file is not found, it is created and initialized with
an 8 word header and the rest is zeroed.  the size of this file
is constant.  there is room in it for storing the 338 blocks
of 256 second intervals that make up one day (includes an extra
slot for the last fraction of an interval that occurs in a day.)

the 256 second block of data in the L1CrisEventCycle struct is then
placed into the day's data file in the location determined by
seconds-since-midnight / 256 (fractions are ignored), e.g.,
interval = 22174 / 256 = 86

this gives us a location in the file to place the L1CrisEventCycle struct:
location for file = (header size) + interval * (L1CrisEventCycle struct size)

putCrisEvents then reads what is already at this location.  if it
is zeroed (from initialization), putCrisEvents writes the passed
L1CrisEventCycle struct into the file and returns a 0.  done.

however, there exists the possibility that, due to some quirk, 2
(or more) passed L1CrisEventCycle structs will lay claim to the same
location.  if this occurs, both the passed L1CrisEventCycle struct
and the L1CrisEventCycle struct already in the file are appended to
an auxiliary file called
/ace/cris/data/C1999.05/C1999.05.23/C1999.05.23.evt.overlap
the contested location in 
/ace/cris/data/C1999.05/C1999.05.23/C1999.05.23.evt
is then filled with char -1s to signal a reader to check the .overlap
file for the multiple structs.  putCrisEvents returns a 1 when
for the first contesting of a particular block and a 2 for every
susequent attempt to write into that particular position.  a 1 is
a signal that something weird happened.  a 2 means something very
strange is going on.

*/

int putCrisEvents(struct L1CrisEventCycle *pDataStruct) {
    unsigned int DATA_TYPE_ID = FILE_CRIS_EVENT_CYCLE;
    char FILE_TYPE_SUFFIX[] = "evt";

	/* declared static so memory allocation occurs only once */
    static char dataBuffer[sizeof(struct L1CrisEventCycle)];

	/* generalized for all cris data structures beyond this point */
	/*     (assumes all such structs have an identical Second1996 field) */

    int fd, intervalNumber, fileInitialized, status;
    char path[256];

	/* this returns a value to intervalNumber as well as path */
    determineDataFilePathAndCreateDirsAsNeeded(pDataStruct->Second1996,
				FILE_TYPE_SUFFIX, path, &intervalNumber, 'C');

    if ((fd = open(path, O_RDWR)) == -1) {
	fileInitialized = 0;
	fd = openOrCreateFileToReadWrite(path);
    }
    else fileInitialized = 1;

    if (!fileInitialized)
	createInitializedFile(fd, dataBuffer, sizeof(dataBuffer), DATA_TYPE_ID);

    status = writeDataStructToFile(fd, intervalNumber, dataBuffer,
				sizeof(dataBuffer), (char *) pDataStruct, path);
    close(fd);
    return status;
}
