#include <stdio.h>
#include <fcntl.h>
#include <time.h>
#include "hdfi.h"
#include "sis.h"
#include "FileTypes.h"
#include "sisPutData.h"
#include "crisSisPutUtilities.h"
#include "utilities.h"

/*

version 2:
	initial version for putSisHighPriorityRates.

putSisHighPriorityRates is passed a pointer to a struct L1SisHighPriorityRate
which it then attempts to archive inside a daily file in the sis data
archive directory tree in this manner:

a base data directory is gotten from the environment variable
L1_SIS_DATA_BASE_DIRECTORY.  if this environment variable has
not been defined, putSisHighPriorityRates exits.  for following examples,
let's say this has been defined as /ace/sis/data.  if /ace/sis/data
does not exist, putSisHighPriorityRates exits.

putSisHighPriorityRates reads the space craft seconds since 1996 and determines
the year, month, day, and seconds since midnight into the day.  for
example's sake, say year = 1999, month = 5 (1-12), day = 23 (1-31),
and seconds since midnight = 22174 (0-86399).

putSisHighPriorityRates then checks for the existence of a directory named
/ace/sis/data/S1999.05/, and if not finding it, creates it.  the
same for the next directory level, /ace/sis/data/S1999.05/S1999.05.23/

putSisHighPriorityRates then checks for the existence of the file
/ace/sis/data/S1999.05/S1999.05.23/S1999.05.23.hp
if this file is not found, it is created and initialized with
an 8 word header and the rest is zeroed.  the size of this file
is constant.  there is room in it for storing the 2700 blocks
of 32 second intervals that make up one day (includes an extra
slot for the last fraction of an interval that occurs in a day.)

the 32 second block of data in the L1SisHighPriorityRate struct is then
placed into the day's data file in the location determined by
seconds-since-midnight / 32 (fractions are ignored), e.g.,
interval = 22174 / 32 = 692

this gives us a location in the file to place the L1SisHighPriorityRate struct:
location for file = (header size) + interval * (L1SisHighPriorityRate struct
size)

putSisHighPriorityRates then reads what is already at this location.  if it
is zeroed (from initialization), putSisHighPriorityRates writes the passed
L1SisHighPriorityRate struct into the file and returns a 0.  done.

however, there exists the possibility that, due to some quirk, 2
(or more) passed L1SisHighPriorityRate structs will lay claim to the same
location.  if this occurs, both the passed L1SisHighPriorityRate struct
and the L1SisHighPriorityRate struct already in the file are appended to
an auxiliary file called
/ace/sis/data/S1999.05/S1999.05.23/S1999.05.23.hp.overlap
the contested location in 
/ace/sis/data/S1999.05/S1999.05.23/S1999.05.23.hp
is then filled with char -1s to signal a reader to check the .overlap
file for the multiple structs.  putSisHighPriorityRates returns a 1 when
for the first contesting of a particular block and a 2 for every
susequent attempt to write into that particular position.  a 1 is
a signal that something weird happened.  a 2 means something very
strange is going on.

if it is determined that the struct to be written to file is
identical to data already in the file at that interval, nothing
is written and a 3 is returned.

*/

int putSisHighPriorityRates(struct L1SisHighPriorityRate *pDataStruct) {
    unsigned int DATA_TYPE_ID = FILE_SIS_HIGH_PRIORITY_RATE;
    char FILE_TYPE_SUFFIX[] = "hp";
    int TIME_INTERVAL = 32;

	/* declared static so memory allocation occurs only once */
    static char dataBuffer[sizeof(struct L1SisHighPriorityRate)];

	/* generalized for all sis data structures beyond this point */
	/*     (assumes all such structs have an identical Second1996 field) */

    int fd, intervalNumber, fileInitialized, status;
    char path[256];
    struct tm *pTime_str;

	/* this returns a value to intervalNumber as well as path */
    determineDataFilePathAndCreateDirsAsNeeded(pDataStruct->Second1996,
				FILE_TYPE_SUFFIX, path, &intervalNumber, 'S');

	/* putSisHighPriorityRates uses 32 second intervals instead of 256 */
    pTime_str = getDateFromSecondsSince1996(pDataStruct->Second1996);
    intervalNumber = (pTime_str->tm_hour * 3600 +
		pTime_str->tm_min * 60 + pTime_str->tm_sec) / TIME_INTERVAL;


    if ((fd = open(path, O_RDWR)) == -1) {
	fileInitialized = 0;
	fd = openOrCreateFileToReadWrite(path);
    }
    else fileInitialized = 1;

    if (!fileInitialized) createInitializedFileWithIntervalN(fd,
		dataBuffer, sizeof(dataBuffer), DATA_TYPE_ID, TIME_INTERVAL);

    status = writeDataStructToFile(fd, intervalNumber, dataBuffer,
				sizeof(dataBuffer), (char *) pDataStruct, path);
    close(fd);
    return status;
}
